/**
 * Copyright 2020 Alibaba Group Holding Limited.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alibaba.maxgraph.common.rpc;

import com.google.common.base.MoreObjects;
import com.google.common.base.Objects;

import org.apache.commons.lang3.StringUtils;

public class RpcAddress {
    private final String host;
    private final int port;

    public RpcAddress(String host, int port) {
        this.host = host;
        this.port = port;
    }

    public RpcAddress(String hostPort) {
        String[] valueList = StringUtils.splitByWholeSeparator(hostPort, ":");
        this.host = valueList[0];
        this.port = Integer.parseInt(valueList[1]);
    }

    public String getHost() {
        return this.host;
    }

    public int getPort() {
        return this.port;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        RpcAddress that = (RpcAddress) o;
        return port == that.port && Objects.equal(host, that.host);
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(host, port);
    }

    @Override
    public String toString() {
        return MoreObjects.toStringHelper(this).add("host", host).add("port", port).toString();
    }
}
